@@ -3,6 +3,7 @@ package MyTest::DBIC::Simple::Product;
 use strict;
 
 use MyTest::DBIC::Simple::Code;
+use MyTest::DBIC::Complex::CodeName;
 use MyTest::DBIC::Simple::Category;
 
 use base 'MyTest::DBIC::Base';
@@ -12,7 +13,11 @@ __PACKAGE__->add_columns(qw(category_id date_created fk1 fk2 fk3 id last_modifie
 __PACKAGE__->set_primary_key('id');
 
 __PACKAGE__->add_relationship('category_id', 'MyTest::DBIC::Simple::Category',
-                            { 'foreign.id' => 'self.category_id' },
-                            { accessor => 'filter' });
+                              { 'foreign.id' => 'self.category_id' },
+                              { accessor => 'filter' });
 
+__PACKAGE__->add_relationship('code_names', 'MyTest::DBIC::Simple::CodeName',
+                              { 'foreign.product_id' => 'self.id' },
+                              { accessor => 'multi' });
+                            
 1;
\ No newline at end of file
